/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.gana.networkprofile;

import br.ufes.inf.decisionelement.gana.networkprofile.configuration.NetworkConfiguration;
import br.ufes.inf.decisionelement.gana.networkprofile.goal.NetworkGoal;
import br.ufes.inf.decisionelement.gana.networkprofile.goal.NetworkProfileGoalException;
import br.ufes.inf.decisionelement.gana.networkprofile.policy.NetworkPolicy;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.json.JsonElement;
import br.ufes.inf.decisionelement.wheelreinventions.json.JsonObject;
import br.ufes.inf.decisionelement.wheelreinventions.json.JsonString;
import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingException;
import java.util.ArrayList;
import java.util.List;

/**
 * No docstring set.
 */
public class NetworkProfile {
    public final List<NetworkConfiguration> networkConfigurations = new ArrayList<>();
    public final List<NetworkPolicy> networkPolicies = new ArrayList<>();
    public final List<NetworkGoal> networkGoals = new ArrayList<>();
    public NetworkProfile() {
    }
    public NetworkProfile(
            List<NetworkConfiguration> networkConfigurations,
            List<NetworkPolicy> networkPolicies,
            List<NetworkGoal> networkGoals
    ) {
        this.networkConfigurations.addAll(networkConfigurations);
        this.networkPolicies.addAll(networkPolicies);
        this.networkGoals.addAll(networkGoals);
    }
    public static NetworkProfile deJson(
            String networkConfigurationsJsonStr,
            String networkPoliciesJsonStr,
            String networkGoalsJsonStr
    ) throws JsonParsingException, MathExpressionException, NetworkProfileGoalException {
        final JsonElement<?> networkConfigurationsJsonEl = JsonElement.parseJson(networkConfigurationsJsonStr);
        final JsonElement<?> networkPoliciesJsonEl = JsonElement.parseJson(networkPoliciesJsonStr);
        final JsonElement<?> networkGoalsJsonEl = JsonElement.parseJson(networkGoalsJsonStr);
        if (!networkConfigurationsJsonEl.isJsonArray()) {
            throw new ClassCastException("JsonElement(NetworkConfigurations) it is not a JsonArray");
        }
        if (!networkPoliciesJsonEl.isJsonArray()) {
            throw new ClassCastException("JsonElement(NetworkPolicies) it is not a JsonArray");
        }
        if (!networkGoalsJsonEl.isJsonArray()) {
            throw new ClassCastException("JsonElement(NetworkGoals) it is not a JsonArray");
        }
        final var networkConfigurationsJsonArr = networkConfigurationsJsonEl.castToArray();
        final var networkPoliciesJsonArr = networkPoliciesJsonEl.castToArray();
        final var networkGoalsJsonArr = networkGoalsJsonEl.castToArray();
        final List<NetworkConfiguration> networkConfigurations = new ArrayList<>();
        final List<NetworkPolicy> networkPolicies = new ArrayList<>();
        final List<NetworkGoal> networkGoals = new ArrayList<>();
        for (var element: networkConfigurationsJsonArr) {
            if (element instanceof JsonObject) {
                networkConfigurations.add(
                        NetworkConfiguration.deJson(
                                (JsonObject) element
                        ));
            } else {
                throw new ClassCastException("JsonElement(NetworkConfiguration) it is not a JsonObject");
            }
        }
        for (var element: networkPoliciesJsonArr) {
            if (element instanceof JsonObject) {
                networkPolicies.add(
                        NetworkPolicy.deJson(
                                (JsonObject) element
                        ));
            } else {
                throw new ClassCastException("JsonElement(NetworkPolicy) it is not a JsonObject");
            }
        }
        for (var element: networkGoalsJsonArr) {
            if (element instanceof JsonString) {
                networkGoals.add(
                        NetworkGoal.deJson(
                                (JsonString) element
                        ));
            } else {
                throw new ClassCastException("JsonElement(NetworkGoal) it is not a JsonString");
            }
        }
        return new NetworkProfile(networkConfigurations, networkPolicies, networkGoals);
    }
}
