/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.operations.TreeFindReplace;
import java.util.Objects;

/**
 * No docstring set.
 * @param <T>
 */
public abstract class MathTree<T> implements MathStateInterface {
    protected final T data;
    public MathTree() throws MathExpressionException {
        data = null;
    }
    public MathTree(T t) throws MathExpressionException {
        data = t;
    }
    @Override
    public int hashCode() {
        return data.hashCode();
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final MathTree<?> other = (MathTree<?>) obj;
        return Objects.equals(this.data, other.data);
    }
    @Override
    public boolean isConsumable() {
        return true;
    }
    @Override
    public boolean isToken() {
        return false;
    }
    @Override
    public boolean isTree() {
        return true;
    }
    @Override
    public String toString() {
        throw new UnsupportedOperationException("This method should be overriden.");
    }
    public abstract MathTree<?> getSimplified() throws MathExpressionException;
    public abstract boolean isConstant();
    public MathTree<?> replace(TreeFindReplace findReplace) throws MathExpressionException {
        if (findReplace.toBeFound.equals(this)) {
            return findReplace.toBeReplaced;
        } else {
            return replaceInner(findReplace);
        }
    }
    public abstract MathTree<?> replaceInner(TreeFindReplace findReplace) throws MathExpressionException;
}
