/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.operations;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTree;
import java.util.Objects;

/**
 * No docstring set.
 */
public class TreeFindReplace {
    public final MathTree<?> toBeFound;
    public final MathTree<?> toBeReplaced;
    public TreeFindReplace(MathTree<?> tf, MathTree<?> tr) {
        toBeFound = tf;
        toBeReplaced = tr;
    }
    public MathTree<?> replaceAt(MathTree<?> tree) throws MathExpressionException {
        return tree.replace(this);
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final TreeFindReplace other = (TreeFindReplace) obj;
        if (!Objects.equals(this.toBeFound, other.toBeFound)) {
            return false;
        }
        return Objects.equals(this.toBeReplaced, other.toBeReplaced);
    }
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 89 * hash + Objects.hashCode(this.toBeFound);
        hash = 89 * hash + Objects.hashCode(this.toBeReplaced);
        return hash;
    }
}
