/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.tokenization;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionParser;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTree;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTreeContext;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * No docstring set.
 */
public class ContextualizationExpressionParser extends MathExpressionParser<String> {
    public ContextualizationExpressionParser() {
        super("$");
    }
    public ContextualizationExpressionParser(String t) {
        super(t);
    }
    @Override
    protected boolean getCharsSequenceSpans() {
        return true;
    }
    @Override
    protected char[] getRecognizedChars() {
        return new char[]{
            'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
            'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
            'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '_', '$'
        };
    }
    @Override
    protected Tuple<MathExpressionParser<?>, String> tryTokenization(String expression) {
        final var dt = this.attemptDefaultTokenization(expression);
        if (dt == null) {
            return null;
        } else {
            return new Tuple<>(new ContextualizationExpressionParser(dt.first), dt.second);
        }
    }
    @Override
    public boolean isConsumable() {
        return false;
    }
    @Override
    protected Tuple<List<MathStateInterface>, Integer>
        parseTokens(List<MathStateInterface> tokens, int i)
            throws MathExpressionException {
        List<MathStateInterface> newTokens = tokens.stream().collect(Collectors.toList());
        if (i <= 0) {
            throw new MathExpressionException("Context requires a preceding token");
        }
        var previousGeneric = newTokens.get(i - 1);
        if (!previousGeneric.isConsumable()) {
            throw new MathExpressionException("Context tries to consume an unconsumable token");
        }
        if (!previousGeneric.isTree()) {
            throw new MathExpressionException("Context tries to consume a non-tree element");
        }
        final var previous = (MathTree) previousGeneric;
        final var current = (ContextualizationExpressionParser) newTokens.get(i);
        newTokens.set(
                i - 1,
                new MathTreeContext(new Tuple<>(
                        Arrays.
                                asList(current.token.split("$")).
                                stream().
                                filter(e -> !e.isEmpty()).
                                sorted().
                                collect(Collectors.toList()),
                        previous
                ))
        );
        newTokens.remove(i);
        return new Tuple<>(newTokens, i - 1);
    }
}
