package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.tokenization;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionParser;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTree;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;
import java.util.List;
import java.util.stream.Collectors;

/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * No docstring set.
 */
public abstract class OperatorExpressionParser extends MathExpressionParser<Character> {
    public OperatorExpressionParser() {
        super();
    }
    public OperatorExpressionParser(Character c) {
        super(c);
    }
    @Override
    protected boolean getCharsSequenceSpans() {
        return false;
    }
    abstract OperatorExpressionParser construct(Character c);
    @Override
    protected Tuple<MathExpressionParser<?>, String> tryTokenization(String expression) {
        final var dt = this.attemptDefaultTokenization(expression);
        if (dt == null) {
            return null;
        } else {
            return new Tuple<>(construct(dt.first.charAt(0)), dt.second);
        }
    }
    protected abstract List<MathStateInterface>
        getLeftTokens(List<MathStateInterface> tokens, int i)
                throws MathExpressionException;
    protected abstract List<MathStateInterface>
        getRightTokens(List<MathStateInterface> tokens, int i)
                throws MathExpressionException;
    protected abstract MathTree
        joinTokens(List<MathStateInterface> leftTokens, List<MathStateInterface> rightTokens)
                throws MathExpressionException;
    @Override
    protected Tuple<List<MathStateInterface>, Integer>
        parseTokens(List<MathStateInterface> tokens, int i)
            throws MathExpressionException {
        List<MathStateInterface> newTokens = tokens.stream().collect(Collectors.toList());
        var leftTokens = getLeftTokens(newTokens, i);
        var rightTokens = getRightTokens(newTokens, i);
        var thisToken = (OperatorExpressionParser) newTokens.get(i);
        newTokens.set(i, thisToken.joinTokens(leftTokens, rightTokens));
        newTokens.removeAll(leftTokens);
        newTokens.removeAll(rightTokens);
        return new Tuple<>(newTokens, i);
    }
}
