/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.tokenization;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTree;
import java.util.Arrays;
import java.util.List;

/**
 * No docstring set.
 */
public abstract class OperatorExpressionParserPlusImpl extends OperatorExpressionParser {
    public OperatorExpressionParserPlusImpl() {
        super();
    }
    public OperatorExpressionParserPlusImpl(Character c) {
        super(c);
    }
    @Override
    protected List<MathStateInterface>
        getLeftTokens(List<MathStateInterface> tokens, int i)
                throws MathExpressionException {
        if (i <= 0) {
            throw new MathExpressionException("Cannot pick tokens at the left of the start");
        }
        return Arrays.asList(tokens.get(i - 1));
    }
    @Override
    protected List<MathStateInterface>
        getRightTokens(List<MathStateInterface> tokens, int i)
                throws MathExpressionException {
        if (i + 1 >= tokens.size()) {
            throw new MathExpressionException("Cannot pick tokens at the right of the end");
        }
        return Arrays.asList(tokens.get(i + 1));
    }
    @Override
    protected MathTree
        joinTokens(List<MathStateInterface> leftTokens, List<MathStateInterface> rightTokens)
                throws MathExpressionException {
        if (leftTokens.size() != 1) {
            throw new MathExpressionException("Operator expects exact one token left side");
        }
        if (rightTokens.size() != 1) {
            throw new MathExpressionException("Operator expects exact one token right side");
        }
        var lt = leftTokens.get(0);
        var rt = rightTokens.get(0);
        if (!(lt instanceof MathTree)) {
            throw new MathExpressionException("Operator expects a tree as left side");
        }
        if (!(rt instanceof MathTree)) {
            throw new MathExpressionException("Operator expects a tree as right side");
        }
        return makeTree(
                (MathTree<?>) lt,
                (MathTree<?>) rt
        );
    }

    protected abstract MathTree makeTree(MathTree lt, MathTree rt) throws MathExpressionException;
}
