/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.tokenization;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionParser;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTreeOperandNested;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;
import java.util.List;
import java.util.stream.Collectors;

/**
 * No docstring set.
 */
public class ParenthesisExpressionParser extends MathExpressionParser<Character> {
    public ParenthesisExpressionParser() {
        super(null);
    }
    public ParenthesisExpressionParser(Character t) {
        super(t);
    }
    @Override
    protected boolean getCharsSequenceSpans() {
        return false;
    }
    @Override
    protected char[] getRecognizedChars() {
        return new char[]{'(', ')'};
    }
    @Override
    protected Tuple<MathExpressionParser<?>, String> tryTokenization(String expression) {
        final var dt = this.attemptDefaultTokenization(expression);
        if (dt == null) {
            return null;
        } else {
            return new Tuple<>(new ParenthesisExpressionParser(dt.first.charAt(0)), dt.second);
        }
    }
    private boolean isClosing() {
        return token == ')';
    }
    @Override
    protected Tuple<List<MathStateInterface>, Integer>
        parseTokens(List<MathStateInterface> tokens, int i)
            throws MathExpressionException {
        List<MathStateInterface> newTokens = tokens.stream().collect(Collectors.toList());
        var tokenStart = newTokens.get(i);
        if (tokenStart instanceof ParenthesisExpressionParser) {
            var pepStart = (ParenthesisExpressionParser) tokenStart;
            if (pepStart.isClosing()) {
                throw new MathExpressionException("Unopened parenthesis");
            }
        } else {
            throw new MathExpressionException("Wrong call");
        }
        int j = i + 1;
        while (j < newTokens.size()) {
            var token = newTokens.get(j);
            if (token instanceof ParenthesisExpressionParser) {
                var pep = (ParenthesisExpressionParser) token;
                if (!pep.isClosing()) {
                    var t = pep.parseTokens(newTokens, j);
                    newTokens = t.first;
                    j = t.second;
                } else {
                    break;
                }
            }
            j++;
        }
        if (j >= newTokens.size()) {
            throw new MathExpressionException("Unclosed parenthesis");
        }
        newTokens.set(
                i,
                new MathTreeOperandNested(
                    MathExpressionParser.makeTreePartial(
                            newTokens.subList(i + 1, j)
                    ).decontextualized()
                )
        );
        newTokens.removeAll(newTokens.subList(i + 1, j + 1));
        return new Tuple<>(newTokens, i);
    }
}
