/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.tokenization;

import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathExpressionException;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.MathStateInterface;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTree;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTreeOperandValueLiteral;
import br.ufes.inf.decisionelement.wheelreinventions.expressionparsing.syntaxtree.MathTreeOperatorBinSub;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;
import java.util.Arrays;
import java.util.List;

/**
 * No docstring set.
 */
public class SubtractionOperatorExpressionParser extends OperatorExpressionParserPlusImpl {
    public SubtractionOperatorExpressionParser() {
        super();
    }
    public SubtractionOperatorExpressionParser(Character c) {
        super(c);
    }
    @Override
    OperatorExpressionParser construct(Character c) {
        return new SubtractionOperatorExpressionParser(c);
    }
    @Override
    protected char[] getRecognizedChars() {
        return new char[]{'-'};
    }
    @Override
    protected List<MathStateInterface>
        getLeftTokens(List<MathStateInterface> tokens, int i)
                throws MathExpressionException {
        if (i <= 0) {
            return Arrays.asList(new MathTreeOperandValueLiteral(0d));
        }
        return Arrays.asList(tokens.get(i - 1));
    }
    @Override
    protected MathTree makeTree(MathTree lt, MathTree rt) throws MathExpressionException {
        return new MathTreeOperatorBinSub(new Tuple<>(lt, rt));
    }
}
