/*
* Copyright 2020 Adler Neves.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package br.ufes.inf.decisionelement.wheelreinventions.json;

import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingException;
import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingUnexpectedEndException;
import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingUnexpectedTokenException;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * No docstring set.
 */
public class JsonArray extends JsonElement<List<JsonElement>> implements List<JsonElement> {
    public JsonArray() {
        super(new ArrayList<>());
    }
    @Override
    public String toString() {
        final var sb = new StringBuilder();
        sb.append('[');
        final var rl = representation.size();
        for (int i = 0; i < rl; i++) {
            final var element = representation.get(i);
            sb.append(element.toString());
            if ((i + 1) < rl) {
                sb.append(',');
            }
        }
        sb.append(']');
        return sb.toString();
    }
    @Override
    protected char[] getHandleChars() {
        return new char[]{'['};
    }
    @Override
    protected Tuple<JsonElement<?>, String> parsePartial(String json) throws JsonParsingException {
        switch (json.charAt(0)) {
            case '[': // array
            {
                var a = new JsonArray();
                String jsonRemaining = json;
                while (true) {
                    if (jsonRemaining.length() <= 0) {
                        throw new JsonParsingUnexpectedEndException("Unterminated JSON array");
                    } else {
                        boolean opens = false;
                        switch (jsonRemaining.charAt(0)) {
                            case '[':
                                opens = true;
                            case ',':
                                jsonRemaining = jsonRemaining.substring(1).stripLeading();
                                if (opens && jsonRemaining.length() > 0 && jsonRemaining.charAt(0) == ']') {
                                    break;
                                }
                                var partialResult = parseJsonPartial(jsonRemaining);
                                a.representation.add(partialResult.first);
                                jsonRemaining = partialResult.second.stripLeading();
                                break;
                            case ']':
                                return new Tuple<>(
                                        a,
                                        jsonRemaining.substring(1)
                                );
                            default:
                                throw new JsonParsingUnexpectedTokenException(
                                        "Expected array token: ',' or ']'"
                                );
                        }
                    }
                }
            }
            default: // unknown
                parsePartialThrower(json);
        }
        throw parseFinalExcGen();
    }

    @Override
    public int size() {
        return representation.size();
    }

    @Override
    public boolean isEmpty() {
        return representation.isEmpty();
    }

    @Override
    public boolean contains(Object arg0) {
        return representation.contains(arg0);
    }

    @Override
    public Iterator<JsonElement> iterator() {
        return representation.iterator();
    }

    @Override
    public Object[] toArray() {
        return representation.toArray();
    }

    @Override
    public <T> T[] toArray(T[] arg0) {
        return representation.toArray(arg0);
    }

    @Override
    public boolean add(JsonElement arg0) {
        return representation.add(arg0);
    }

    @Override
    public boolean remove(Object arg0) {
        return representation.remove(arg0);
    }

    @Override
    public boolean containsAll(Collection<?> arg0) {
        return representation.containsAll(arg0);
    }

    @Override
    public boolean addAll(Collection<? extends JsonElement> arg0) {
        return representation.addAll(arg0);
    }

    @Override
    public boolean addAll(int arg0, Collection<? extends JsonElement> arg1) {
        return representation.addAll(arg0, arg1);
    }

    @Override
    public boolean removeAll(Collection<?> arg0) {
        return representation.removeAll(arg0);
    }

    @Override
    public boolean retainAll(Collection<?> arg0) {
        return representation.retainAll(arg0);
    }

    @Override
    public void clear() {
        representation.clear();
    }

    @Override
    public JsonElement get(int arg0) {
        return representation.get(arg0);
    }

    @Override
    public JsonElement set(int arg0, JsonElement arg1) {
        return representation.set(arg0, arg1);
    }

    @Override
    public void add(int arg0, JsonElement arg1) {
        representation.add(arg0, arg1);
    }

    @Override
    public JsonElement remove(int arg0) {
        return representation.remove(arg0);
    }

    @Override
    public int indexOf(Object arg0) {
        return representation.indexOf(arg0);
    }

    @Override
    public int lastIndexOf(Object arg0) {
        return representation.lastIndexOf(arg0);
    }

    @Override
    public ListIterator<JsonElement> listIterator() {
        return representation.listIterator();
    }

    @Override
    public ListIterator<JsonElement> listIterator(int arg0) {
        return representation.listIterator(arg0);
    }

    @Override
    public List<JsonElement> subList(int arg0, int arg1) {
        return representation.subList(arg0, arg1);
    }
}
