/*
 * Copyright 2020 Adler Neves.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package br.ufes.inf.decisionelement.wheelreinventions.json;

import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingException;
import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingUnexpectedSymbolException;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;

/**
 * No docstring set.
 */
public class JsonBoolean extends JsonElement<Boolean> {
    public JsonBoolean() {
        super(false);
    }
    @Override
    protected char[] getHandleChars() {
        return new char[]{'t', 'f'};
    }
    @Override
    public String toString() {
        return representation ? "true" : "false";
    }
    @Override
    protected Tuple<JsonElement<?>, String> parsePartial(String json) throws JsonParsingException {
        switch (json.charAt(0)) {
                case 't': // boolean true
                    if (!json.startsWith("true")) {
                        throw new JsonParsingUnexpectedSymbolException(
                                "Did not find \"true\" on a object that starts with \'t\'."
                        );
                    } else {
                        var b = new JsonBoolean();
                        b.representation = true;
                        return new Tuple<>(
                                b,
                                json.substring(4)
                        );
                    }
                case 'f': // boolean false
                    if (!json.startsWith("false")) {
                        throw new JsonParsingUnexpectedSymbolException(
                                "Did not find \"false\" on a object that starts with \'f\'."
                        );
                    } else {
                        var b = new JsonBoolean();
                        b.representation = false;
                        return new Tuple<>(
                                b,
                                json.substring(5)
                        );
                    }
                default: // unknown
                    parsePartialThrower(json);
        }
        throw parseFinalExcGen();
    }
}
