/*
* Copyright 2020 Adler Neves.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package br.ufes.inf.decisionelement.wheelreinventions.json;

import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingException;
import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingUnexpectedSymbolException;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;

/**
 * No docstring set.
 * @param <T>
 */
public class JsonNull<T> extends JsonElement<T> {
    private static JsonNull<?> instance = null;
    public static JsonNull<?> getInstance() {
        if (instance == null) {
            instance = new JsonNull();
        }
        return instance;
    }
    public JsonNull() {
        representation = null;
    }
    @Override
    public String toString() {
        return "null";
    }
    @Override
    protected char[] getHandleChars() {
        return new char[]{'n'};
    }
    @Override
    protected Tuple<JsonElement<?>, String> parsePartial(String json) throws JsonParsingException {
        switch (json.charAt(0)) {
            case 'n': // null literal
                if (!json.startsWith("null")) {
                    throw new JsonParsingUnexpectedSymbolException(
                            "Did not find \"null\" on a object that starts with \'n\'."
                    );
                } else {
                    return new Tuple<>(
                            getInstance(),
                            json.substring(4)
                    );
                }
            default: // unknown
                parsePartialThrower(json);
        }
        throw parseFinalExcGen();
    }
}
