/*
* Copyright 2020 Adler Neves.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package br.ufes.inf.decisionelement.wheelreinventions.json;

import br.ufes.inf.decisionelement.wheelreinventions.json.exceptions.JsonParsingException;
import br.ufes.inf.decisionelement.wheelreinventions.positionalcollections.Tuple;

/**
 * No docstring set.
 */
public class JsonNumeric extends JsonElement<Double> {
    private static final char[] NUMERIC_VALS = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.'};
    public JsonNumeric() {
        super(0d);
    }
    @Override
    protected char[] getHandleChars() {
        return NUMERIC_VALS;
    }
    @Override
    public String toString() {
        return representation.toString();
    }
    @Override
    protected Tuple<JsonElement<?>, String> parsePartial(String json) throws JsonParsingException {
        switch (json.charAt(0)) {
            case '0': // number
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
            case '.':
            {
                var sb = new StringBuilder();
                int i = 0;
                for (; i < json.length(); i++) {
                    char c = json.charAt(i);
                    boolean f = false;
                    for (char n : NUMERIC_VALS) {
                        if (c == n) {
                            f = true;
                            break;
                        }
                    }
                    if (f) {
                        sb.append(c);
                    } else {
                        break;
                    }
                }
                var n = new JsonNumeric();
                n.representation = Double.parseDouble(sb.toString());
                return new Tuple<>(n, json.substring(i));
            }
            default: // unknown
                parsePartialThrower(json);
        }
        throw parseFinalExcGen();
    }
}
